//-----------------------------------------------------------------
// Consider the example of a drunkards's walk for n blocks. 
// The absorbing states are S0 and Sn, the transient states 
// are si, i=1,...n-1, and the non-zero transition probabilities 
// are pi,i+1=pi,i-1=1/2, and p00=0nn=1. 
// Prove that the expectation time to absorption, 
// starting at state Si, is equal to i(n-i) 
//-----------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define N_jk 10000


void jk(double &avg, double &err, double* jk, int N){
  avg = 0.0;
  double tmp = 0.0;
  double jk_sum = 0.0;
  for(int i=0; i<N; i++){
    jk_sum += jk[i];
  }
  avg = jk_sum/(double)N;

  for(int i=0; i<N; i++){
    tmp += pow(avg - jk[i], 2);
  }
  err = tmp * (double)(N-1) / (double)N ;
  err = sqrt(err);
}

int irand(){
  int ran = (rand()%2) * 2 - 1;
  return ran;
}

int main(int argc, char* argv[]){

  if(argc != 2){
    printf("input n\n");
    exit(0);
  }

  int n = atoi(argv[1]);


  int **Z = new int*[n+1];

  for(int i = 0 ; i < n+1 ; i++){
    Z[i] = new int[N_jk];

  }
  // initialization
  for(int i = 0 ; i < n+1 ; i++){
    for(int j = 0 ; j < N_jk ; j++){
      Z[i][j] = 0;
    }
  }

  for(int ijk = 0 ; ijk < N_jk ; ijk++){
      for(int i = 1 ; i < n ; i++){
        int x = i; // starting point
        int time = 0;
        while(1){
          x += irand();
          time++;
          if(x == 0 || x == n)break;
        }
        Z[i][ijk] += time;
      }
  }
  
  // jankknife resampling
  double **jk_avg = new double*[n+1];
  for(int i = 0 ; i < n+1 ; i++){
    jk_avg[i] = new double[N_jk];
    for(int j = 0 ; j < N_jk ; j++){
      jk_avg[i][j] = 0.0;
    }
  }

  for(int i = 0 ; i < n+1 ; i++){
    double tmp = 0.0;
    for(int j = 0 ; j < N_jk ; j++){
      tmp += (double)Z[i][j];
    }
    for(int del = 0 ; del < N_jk ; del++){
      jk_avg[i][del] = tmp - (double)Z[i][del];
      jk_avg[i][del] /= (double)(N_jk-1);
    }
  }
  
  double *avg = new double[n+1];
  double *err = new double[n+1];
  for(int i = 0 ; i < n+1 ; i++){
    jk(avg[i], err[i], jk_avg[i], N_jk);
  }

  // save the result as file
  FILE *fp_dat;
  char fname_dat[1024];
  sprintf(fname_dat,"data.dat");
  fp_dat = fopen(fname_dat, "w");

  for(int i = 0 ; i < n+1 ; i++){
    if(avg[i] != 0.0){
      fprintf(stdout, "%d %lf %lf\n", i, avg[i], err[i]);
      fprintf(fp_dat, "%d %lf %lf\n", i, avg[i], err[i]);
    }
  }

  for(int i = 0 ; i < n+1 ; i++){ 
    delete[] Z[i];
    delete[] jk_avg[i];
  }
  delete[] Z;
  delete[] jk_avg;
  delete[] avg;
  delete[] err;

  return 0;
}
