//-----------------------------------------------------------------
// Perform a simulation of the symmetric random walk and 
// reproduce the above probability for various y > 0, 
// with n = 100 fixed.
//-----------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define n 100
#define Njk 10

void jk(double &avg, double &err, double* jk, int N){
  avg = 0.0;
  double tmp = 0.0;
  double jk_sum = 0.0;
  for(int i=0; i<N; i++){
    jk_sum += jk[i];
  }
  avg = jk_sum/(double)N;

  for(int i=0; i<N; i++){
    tmp += pow(avg - jk[i], 2);
  }
  err = tmp * (double)(N-1) / (double)N ;
  err = sqrt(err);
}

int irand(){
  int ran = (rand()%2) * 2 - 1;
  return ran;
}

int main(int argc, char* argv[]){

  int *S = new int[n+1];// S[0]~S[100]
  int **Z = new int*[n+1];// counter for allowed path
  int **T = new int*[n+1];// counter for all path

  for(int i = 0 ; i < n+1 ; i++){
    Z[i] = new int[Njk];
    T[i] = new int[Njk];

  }
  // initialization
  for(int i = 0 ; i < n+1 ; i++){
    for(int j = 0 ; j < Njk ; j++){
      Z[i][j] = 0;
      T[i][j] = 0;
    }
  }

  const int N_Try = 10000;
  // start simulation
  for(int icon = 0 ; icon < Njk ; icon++){

    for(int itry = 0 ; itry < N_Try ; itry++){
      S[0] = 0; // start at origin

      // random walk
      for(int i = 0 ; i < n ; i++){
        S[i+1] = S[i] + irand();
      }

      // check whether walker returned to the origin or not.
      int FLAG = 1;
      for(int i = 1 ; i <= n ; i++){
        if(S[i] <=0)
        FLAG = 0; // if it pass the origin, FLAG = 0.
      }

      int y = S[n]; // final position is y
      if(y > 0){
        T[y][icon]++; // count the total number of paths to y. 
      }

      if(FLAG){
        Z[y][icon]++;// count the number of paths to y which does not pass the origin. 
      }

    }
  }

  // jankknife resampling
  double **jk_avg = new double*[n+1];
  for(int i = 0 ; i < n+1 ; i++){
    jk_avg[i] = new double[Njk];
    for(int j = 0 ; j < Njk ; j++){
      jk_avg[i][j] = 0.0;
    }
  }

  for(int i = 0 ; i < n+1 ; i++){
    for(int del = 0 ; del < Njk ; del++){
      for(int j = 0 ; j < Njk ; j++){
        if(j == del) continue;

        if(Z[i][j] != 0 && T[i][j] != 0){
          jk_avg[i][del] += (double)Z[i][j]/T[i][j];
        }
      }
      jk_avg[i][del] /= (double)(Njk-1);
    }
  }

  double *avg = new double[n+1];
  double *err = new double[n+1];
  for(int i = 0 ; i < n+1 ; i++){
    jk(avg[i], err[i], jk_avg[i], Njk);
  }


  // save the result as file
  FILE *fp_dat;
  char fname_dat[1024];
  sprintf(fname_dat,"data.dat");
  fp_dat = fopen(fname_dat, "w");

  for(int i = 0 ; i < n+1 ; i++){
    if(avg[i] != 0.0){
      fprintf(stdout, "%d %lf %lf\n", i, avg[i], err[i]);
      fprintf(fp_dat, "%d %lf %lf\n", i, avg[i], err[i]);
    }
  }

  delete[] S;
  delete[] Z;
  delete[] T;

  return 0;
}
